	list p=16f628
;**************************************************************
;*
;*  	Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 -
;*		1 -
;*		2 -
;*		3 -
;*		4 -
;*		5 -
;*		6 -
;*		7 -
;*
;*	PORTB:	0 > LCD Display E
;*		1 -
;*		2 > LCD Display RS
;*		3 > LCD Display R/W
;*		4 - LCD Display D4
;*		5 - LCD Display D5
;*		6 - LCD Display D6
;*		7 - LCD Display D7
;*
;**************************************************************
;
; sprut (zero) Bredendiek 02/2004  mod 06/2005
;
; Ausgabe selbst entworfener Symbole am Dotmatrixdisplay
;  Takt : intern 4 MHz
;
;
;**************************************************************
; Includedatei fr den 16F628 einbinden
	#include <P16f628.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen
; Power on Timer, kein Watchdog, interner Takt
	__CONFIG	_PWRTE_ON & _WDT_OFF & _INTRC_OSC_CLKOUT & _BODEN_OFF & _LVP_OFF

;**************************************************************
; Konstanten

; Variablennamen vergeben
loops		equ	0x22		; timer fr wait
loops2		equ	0x23		; timer fr wait
LcdStatus	equ	0x24		; fuer LcdBusy
LcdDaten	equ	0x25	
CHARCTR		equ	0x26		; fuer CGRAM-Routinen		
TEMP		equ	0x27
pos		equ	0x28		; Displayposition fuer Balkengrafik

; Bezeichner fr LCD-Pins
#define	LcdE		PORTB,0		; enable Lcd
#define	LcdRw		PORTB,3		; read Lcd
#define	LcdRs		PORTB,2		; 1=Daten Lcd;  0=control Lcd
#define LcdPort		PORTB		; Datenbus des LCD (obere 4 Bit)

; Konstanten fuer Grundeinstellungen
Ini_con		Equ	B'00000000'	; TMR0 -> Interupt disable
Ini_opt		Equ	B'00000010'	; pull-up

;**************************************************************
; los gehts mit dem Programm

	org 	0
	goto	Init	

;**************************************************************
; das sind die 8 Symbole, die in den CGRAM des Displays kopiert werden muessen
CGRAMTABLE
	MOVF	CHARCTR,W
	ADDWF	PCL,F		;TABLE FOR CUSTOM CGRAM
	;CYRILLIC L , Tabellenoffset 0
	RETLW	B'01111'	;  ****
	RETLW	B'00101'	;   * *
	RETLW	B'00101'	;   * *
	RETLW	B'00101'	;   * *
	RETLW	B'00101'	;   * *
	RETLW	B'10101'	; * * *
	RETLW	B'01001'	;  *  *
	RETLW	B'00000'	; 
	;CYRILLIC I , Tabellenoffset 8
	RETLW	B'10001'	; *   *
	RETLW	B'10001'	; *   *
	RETLW	B'10011'	; *  **
	RETLW	B'10101'	; * * *
	RETLW	B'11001'	; **  *
	RETLW	B'10001'	; *   *
	RETLW	B'10001'	; *   *
	RETLW	B'00000'	;
	;CYRILLIC G , Tabellenoffset 16
	RETLW	B'11111'	; *****
	RETLW	B'10001'	; *   *
	RETLW	B'10000'	; *   
	RETLW	B'10000'	; *   
	RETLW	B'10000'	; *   
	RETLW	B'10000'	; *   
	RETLW	B'10000'	; *   
	RETLW	B'00000'	;
	;| , Tabellenoffset 24
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	;|| , Tabellenoffset 32
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	;||| , Tabellenoffset 40
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***	
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***	
	RETLW	B'11100'	; ***	
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***
	;|||| , Tabellenoffset 48
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	;|||| , Tabellenoffset 54
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
;**************************************************************

Init
; Port B auf Ausgabe stellen
	bsf     STATUS, RP0	; Bank 1
	movlw   Ini_opt     	; pull-up on
	movwf   OPTION_REG 
	movlw	B'00000010'	; PortB alle outputs auer RB1
	movwf	TRISB
	bcf     STATUS, RP0	; Bank 0
	clrf	PORTB

	movlw   Ini_con     	; Interupt disable
	movwf   INTCON   

;Display initialisieren
	call	InitLcd

;mal sehen, ob das Display nun funktioniert, OK anzeigen
	movlw	'O'			; A
	CALL	OutLcdDaten
	movlw	'K'			; A
	CALL	OutLcdDaten

; 1 Sekunde warten
	CALL	Wait250
	CALL	Wait250
	CALL	Wait250
	CALL	Wait250

;CGRAM Adresse auf 0x00 setzen
	MOVLW   B'00000000'
	CALL	SETCGADDR

;CGRAM mit den neuen Symbolen laden
	MOVLW	.0			;0: CYR L
	CALL	CGRAMLOAD
	MOVLW	.8			;1: CYR I
	CALL	CGRAMLOAD
	MOVLW	.16			;2: CYR G
	CALL	CGRAMLOAD
	MOVLW	.24			;3: |
	CALL	CGRAMLOAD
	MOVLW	.32			;4: ||
	CALL	CGRAMLOAD
	MOVLW	.40			;5: |||
	CALL	CGRAMLOAD
	MOVLW	.48			;6: ||||
	CALL	CGRAMLOAD
	MOVLW	.56			;7: |||||
	CALL	CGRAMLOAD

HelloWorld
;DDRAM ADRESSE SETZEN
	MOVLW   B'00000000'
	CALL	SETDDADDR  		; Kursor an den displayanfang setzen

;TEXTAUSGABE: Hallo World
	movlw	0x02			; G
	CALL	OutLcdDaten
	movlw	'A'			; A
	CALL	OutLcdDaten
	movlw	0x00			; L
	CALL	OutLcdDaten
	movlw	0x00			; L
	CALL	OutLcdDaten
	movlw	'O'			; O
	CALL	OutLcdDaten
	movlw	' '			; 
	CALL	OutLcdDaten
	movlw	'M'			; M
	CALL	OutLcdDaten
	movlw	0x01			; I
	CALL	OutLcdDaten
	movlw	'P'			; P
	CALL	OutLcdDaten

	CALL	Wait250
	CALL	Wait250
	CALL	Wait250
	CALL	Wait250
	

; Balkengrafik als Endlosschleife
loop
	; Die Grafik beginnt auf der zehnten Displaystelle
	movlw	B'00001010'
	movwf	pos

	; wachsender Balken
	call	grow			; auf Stelle 10
	incf	pos,f
	call	grow			; auf Stelle 11
	incf	pos,f
	call	grow			; auf Stelle 12
	incf	pos,f
	call	grow			; auf Stelle 13
	incf	pos,f
	call	grow			; auf Stelle 14
	; schrumpfender Balken
	call	disgrow			; auf Stelle 14
	decf	pos,f
	call	disgrow			; auf Stelle 13
	decf	pos,f
	call	disgrow			; auf Stelle 12
	decf	pos,f
	call	disgrow			; auf Stelle 11
	decf	pos,f
	call	disgrow			; auf Stelle 10

	goto	loop			; eine Endlosschleife

;*****************************************************
; Ende des Hauptprogramms
;*****************************************************


;*****************************************************
; Routinen fuer Balkengrafik
;*****************************************************
; wachsender Balken innerhalb einer Displaystelle
grow
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x03			; |
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x04			; ||
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x05			; |||
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x06			; ||||
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x07			; |||||
	CALL	OutLcdDaten
	CALL	Wait100
	return


;*****************************************************
; schrumpfender Balken innerhalb einer Displaystelle
disgrow
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x06			; ||||
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x05			; |||
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x04			; ||
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	0x03			; |
	CALL	OutLcdDaten
	CALL	Wait100
	MOVFW   pos
	CALL	SETDDADDR  		; REPOSITION CURSOR
	movlw	' '			; 
	CALL	OutLcdDaten
	CALL	Wait100
	return



;*****************************************************
;+++LCD-Routinen**************************************
;*****************************************************
;LCD initialisieren
InitLcd
	call	Wait250		; 250 ms Pause nach dem Einschalten

	movlw	B'00110000'	; 1
	movwf	LcdPort
	bsf	LcdE
	nop	
	bcf	LcdE
	
	movlw	D'50'		; 50 ms Pause
	movwf	loops
	call	WAITms
	
	movlw	B'00110000'	; 2
	call	Control8Bit
	movlw	B'00110000'	; 3
	call 	Control8Bit
	movlw	B'00100000'	; 4
	call 	Control8Bit

	movlw	B'00000001'	; lschen und cusor home
	call	OutLcdControl	
	movlw	B'00101000'	; 5 function set, 4-bit  2-zeilig,  5x7
	call	OutLcdControl	
	movlw	B'00001000'	; 6 display off
	call	OutLcdControl
	movlw	B'00000110'	; 7 entry mode, increment, disable display-shift
	call	OutLcdControl
	movlw	B'00000011'	; 8 cursor home, cursor home
	call	OutLcdControl
	movlw	B'00001100'	; 9 display on, Kursor aus , Blinken aus
	call	OutLcdControl
	return

;*****************************************************
; ein Steuerbyte 8-bittig bertragen
Control8Bit
	movwf	LcdPort
	bsf	LcdE
	nop
	bcf	LcdE
	movlw	D'10'
	movwf	loops
	call 	WAITms
	return

;*****************************************************
; darauf warten, da das Display bereit zur Datenannahme ist
LcdBusy
        bsf     STATUS, RP0	; make Port B4..7 input
	movlw	B'11110000'
	iorwf   TRISB, f 
        bcf     STATUS, RP0
BusyLoop		
	bcf	LcdRs
	bsf	LcdRw		; Lesen
	bsf	LcdE
	nop
	movf	LcdPort, w
	movwf	LcdStatus
	bcf	LcdE
	nop
	bsf	LcdE		; Enable
	nop
	bcf	LcdE
	btfsc	LcdStatus, 7	; teste bit 7
	goto	BusyLoop
	bcf	LcdRw
        bsf     STATUS, RP0	; make Port B4..7 output
	movlw	B'00001111'
	andwf   TRISB, f    
        bcf     STATUS, RP0
	return	

;*****************************************************
; aus W ein Byte mit Steuerdaten zum Display bertragen
OutLcdControl
	movwf	LcdDaten
	call	LcdBusy
	movf	LcdDaten, w
	andlw	H'F0'
	movwf	LcdPort		; Hi-teil Daten schreiben
	bsf	LcdE
	nop
	bcf	LcdE		; Disable LcdBus
	swapf	LcdDaten, w
	andlw	H'F0'
	movwf	LcdPort		; Lo-teil Daten schreiben
	bsf	LcdE
	nop
	bcf	LcdE		; Disable LcdBus
	return

;*****************************************************
; aus W ein Datenbyte zum Display bertragen
OutLcdDaten
	movwf	LcdDaten
	call	LcdBusy
	movf	LcdDaten, w
	andlw	H'F0'
	movwf	LcdPort		; Hi-teil Daten schreiben
	bsf	LcdRs		; Daten
	bsf	LcdE		; Enable LcdBus
	nop
	bcf	LcdE		; Disable LcdBus	
	swapf	LcdDaten, w
	andlw	H'F0'
	movwf	LcdPort		; Lo-teil Daten schreiben
	bsf	LcdRs		; Daten
	bsf	LcdE
	nop
	bcf	LcdE		; Disable LcdBus	
	bcf	LcdRs		;
	return

;*****************************************************
; Es folgen die Routinen fue CGRAM
;*****************************************************

;*****************************************************
;SET CGRAM TEST ROUTINE
;IN: W=CGRAM LOCATION
;DESTROYS; W, CHARCTR
;*****************************************************
SETCGADDR
	bcf	LcdRs 		; INSTR REG
	bcf	LcdRw		; WRITE
	ANDLW	B'00111111'	; CGRAM CHOPOFF
	IORLW	B'01000000' 	; CGRAM AREA
	CALL	OutLcdControl
	RETURN


;*****************************************************
;SET DDRAM TEST ROUTINE
;IN: W=DDRAM LOCATION
;DESTROYS; W, CHARCTR
;*****************************************************
SETDDADDR
	bcf	LcdRs 		; INSTR REG
	bcf	LcdRw		; WRITE
	ANDLW	B'01111111'	; DDRAM CHOPOFF
	IORLW	B'10000000' 	; DDRAM AREA
	CALL	OutLcdControl
	RETURN


;*****************************************************
;CGRAM CHARACTER LOAD ROUTINE
;IN: W=NUMBER
;DESTROYS; W, CHARCTR
;*****************************************************
CGRAMLOAD
	BSF	LcdRs 		; INSTR REG
	bcf	LcdRw		; WRITE
	MOVWF	CHARCTR		; CHARACTER COUNTER
	MOVLW	0X08
	MOVWF	TEMP		; 8 BYTES TO SEND
			
CGRAMSUB1
	CALL	CGRAMTABLE	;LOAD A CHAR FROM ROM TBL
	CALL	OutLcdDaten
	INCF	CHARCTR,F	;INC CHAR COUNTER
	DECFSZ	TEMP,F
	GOTO	CGRAMSUB1
	RETURN


;*****************************************************
; Warteschleifen
;*****************************************************
; Warteschleife 250 ms
Wait250
	movlw	D'250'		; 250 ms Pause
	movwf	loops
	goto	Wai
	
; Warteschleife 100 ms
Wait100
	movlw	D'100'		; 100 ms Pause
	movwf	loops	

; Warteschleife, Anzahl der Millisekunden steht in loops
Wai
WAITms
	movlw   .110           ; Zeitkonstante fr 1ms
        movwf   loops2
Wai2    nop                    ; 
        nop
        nop
        nop
        nop
        nop
        decfsz  loops2, F      ; 1 ms vorbei?
        goto    Wai2           ; nein, noch nicht
                               ;
        decfsz  loops, F       ; alle ms vorbei?
        goto    Wai            ; nein, noch nicht
        retlw   0              ; das Warten hat ein Ende

;*****************************************************

	end



